/**
 * Create an HTML extension for `micromark` to support GitHub tables when
 * serializing to HTML.
 *
 * @returns {HtmlExtension}
 *   Extension for `micromark` that can be passed in `htmlExtensions` to
 *   support GitHub tables when serializing to HTML.
 */
export function gfmTableHtml(): HtmlExtension;
import type { HtmlExtension } from 'micromark-util-types';
//# sourceMappingURL=html.d.ts.map