// @ts-check

import { homepage, version } from "./constants.mjs";

import md001 from "./md001.mjs";
import md003 from "./md003.mjs";
import md004 from "./md004.mjs";
import md005 from "./md005.mjs";
import md007 from "./md007.mjs";
import md009 from "./md009.mjs";
import md010 from "./md010.mjs";
import md011 from "./md011.mjs";
import md012 from "./md012.mjs";
import md013 from "./md013.mjs";
import md014 from "./md014.mjs";
import md018 from "./md018.mjs";
import md019md021 from "./md019-md021.mjs";
const [ md019, md021 ] = md019md021;
import md020 from "./md020.mjs";
import md022 from "./md022.mjs";
import md023 from "./md023.mjs";
import md024 from "./md024.mjs";
import md025 from "./md025.mjs";
import md026 from "./md026.mjs";
import md027 from "./md027.mjs";
import md028 from "./md028.mjs";
import md029 from "./md029.mjs";
import md030 from "./md030.mjs";
import md031 from "./md031.mjs";
import md032 from "./md032.mjs";
import md033 from "./md033.mjs";
import md034 from "./md034.mjs";
import md035 from "./md035.mjs";
import md036 from "./md036.mjs";
import md037 from "./md037.mjs";
import md038 from "./md038.mjs";
import md039 from "./md039.mjs";
import md040 from "./md040.mjs";
import md041 from "./md041.mjs";
import md042 from "./md042.mjs";
import md043 from "./md043.mjs";
import md044 from "./md044.mjs";
import md045 from "./md045.mjs";
import md046 from "./md046.mjs";
import md047 from "./md047.mjs";
import md048 from "./md048.mjs";
import md049md050 from "./md049-md050.mjs";
const [ md049, md050 ] = md049md050;
import md051 from "./md051.mjs";
import md052 from "./md052.mjs";
import md053 from "./md053.mjs";
import md054 from "./md054.mjs";
import md055 from "./md055.mjs";
import md056 from "./md056.mjs";
import md058 from "./md058.mjs";
import md059 from "./md059.mjs";

const rules = [
  md001,
  // md002: Deprecated and removed
  md003,
  md004,
  md005,
  // md006: Deprecated and removed
  md007,
  md009,
  md010,
  md011,
  md012,
  md013,
  md014,
  md018,
  md019,
  md020,
  md021,
  md022,
  md023,
  md024,
  md025,
  md026,
  md027,
  md028,
  md029,
  md030,
  md031,
  md032,
  md033,
  md034,
  md035,
  md036,
  md037,
  md038,
  md039,
  md040,
  md041,
  md042,
  md043,
  md044,
  md045,
  md046,
  md047,
  md048,
  md049,
  md050,
  md051,
  md052,
  md053,
  md054,
  md055,
  md056,
  // md057: See https://github.com/markdownlint/markdownlint
  md058,
  md059
];
for (const rule of rules) {
  const name = rule.names[0].toLowerCase();
  // eslint-disable-next-line dot-notation
  rule["information"] = new URL(`${homepage}/blob/v${version}/doc/${name}.md`);
}
export default rules;
