import type { ConfigurationStrict } from "./configuration-strict.d.ts";

export interface Configuration extends ConfigurationStrict {
  /**
   * Index signature for arbitrary custom rules.
   */
  [k: string]: unknown;
}
