/**
 * @param {Options | null | undefined} [options={}]
 *   Configuration (default: `{}`).
 * @returns {Construct}
 *   Construct.
 */
export function mathText(options?: Options | null | undefined): Construct;
import type { Options } from 'micromark-extension-math';
import type { Construct } from 'micromark-util-types';
