/**
 * Create an extension for `micromark` to support directives when serializing
 * to HTML.
 *
 * @param {HtmlOptions | null | undefined} [options={}]
 *   Configuration (default: `{}`).
 * @returns {HtmlExtension}
 *   Extension for `micromark` that can be passed in `htmlExtensions`, to
 *   support directives when serializing to HTML.
 */
export function directiveHtml(options?: HtmlOptions | null | undefined): HtmlExtension;
import type { HtmlOptions } from 'micromark-extension-directive';
import type { HtmlExtension } from 'micromark-util-types';
//# sourceMappingURL=html.d.ts.map