/**
 * Parse HTML character references.
 *
 * @param {string} value
 * @param {Readonly<Options> | null | undefined} [options]
 */
export function parseEntities(value: string, options?: Readonly<Options> | null | undefined): string;
import type { Options } from '../index.js';
//# sourceMappingURL=index.d.ts.map