/**
 * Create an extension for `micromark` to enable directive syntax.
 *
 * @returns {Extension}
 *   Extension for `micromark` that can be passed in `extensions`, to
 *   enable directive syntax.
 */
export function directive(): Extension;
import type { Extension } from 'micromark-util-types';
//# sourceMappingURL=syntax.d.ts.map