/**
 * Tokenize subcontent.
 *
 * @param {Array<Event>} eventsArray
 *   List of events.
 * @returns {boolean}
 *   Whether subtokens were found.
 */
export function subtokenize(eventsArray: Array<Event>): boolean;
export { SpliceBuffer } from "./lib/splice-buffer.js";
import type { Event } from 'micromark-util-types';
//# sourceMappingURL=index.d.ts.map