/** @type {InitialConstruct} */
export const document: InitialConstruct;
/**
 * Construct and its state.
 */
export type StackItem = [Construct, ContainerState];
import type { InitialConstruct } from 'micromark-util-types';
import type { Construct } from 'micromark-util-types';
import type { ContainerState } from 'micromark-util-types';
//# sourceMappingURL=document.d.ts.map